@@ -37,7 +37,8 @@ List GLMcat(Formula formula,
             CharacterVector ref_category,
             double freedom_degrees,
             std::string threshold,
-            Eigen::VectorXd beta_init){
+            NumericVector beta_init){
+  // Eigen::VectorXd beta_init){
 
   LogicalVector is_na_ref1 = is_na(categories_order);
   if(is_na_ref1[0]){ // categories order is not given
@@ -80,8 +81,9 @@ List GLMcat(Formula formula,
   Eigen::VectorXd BETA3 = BETA2;
 
   Eigen::MatrixXd BETA = BETA2;
-  if(beta_init.size() >= 2 ){
-    BETA = beta_init;
+  if(beta_init.length() >= 2 ){
+    // BETA = beta_init;
+    BETA = as<Eigen::Map<Eigen::VectorXd> >(beta_init);
   }
   //
   int iteration = 0;
@@ -101,7 +103,7 @@ List GLMcat(Formula formula,
   MatrixXd cov_beta;
   VectorXd Std_Error;
   double LogLik;
-  MatrixXd pi_ma(N, K);
+  MatrixXd pi_ma(N, Q);
   MatrixXd F_i_final = MatrixXd::Zero(BETA.rows(), BETA.rows());
 
   // for (int iteration=1; iteration < 18; iteration++){
@@ -119,12 +121,12 @@ List GLMcat(Formula formula,
       X_M_i = X_EXT.block(i*Q , 0 , Q , X_EXT.cols());
       Y_M_i = Y_init.row(i);
       eta = X_M_i * BETA;
-
+      VectorXd eta1 = eta;
       if(ratio == "reference"){
         ReferenceF ref;
         if(distribution == "logistic"){
-          pi = ref.inverse_logistic(eta);
-          D = ref.inverse_derivative_logistic(eta);
+          pi = ref.inverse_logistic(eta1);
+          D = ref.inverse_derivative_logistic(eta1);
         }else if(distribution == "normal"){
           pi = ref.inverse_normal(eta);
           D = ref.inverse_derivative_normal(eta);
@@ -168,7 +170,6 @@ List GLMcat(Formula formula,
         }
       }else if(ratio == "sequential"){
         SequentialR seq;
-        // Vector pi depends on selected distribution
         if(distribution == "logistic"){
           pi = seq.inverse_logistic(eta);
           D = seq.inverse_derivative_logistic(eta);
@@ -192,7 +193,6 @@ List GLMcat(Formula formula,
         }
       }else if(ratio == "cumulative"){
         CumulativeR cum;
-        // Vector pi depends on selected distribution
         if(distribution == "logistic"){
           pi = cum.inverse_logistic(eta);
           D = cum.inverse_derivative_logistic(eta);
@@ -223,22 +223,34 @@ List GLMcat(Formula formula,
       // Rcout << "Cov_i.determinant()" << std::endl;
       // Rcout << Cov_i.determinant() << std::endl;
       W_in = D * Cov_i.inverse();
-      // Rcout << "W_in" << std::endl;
-      // Rcout << W_in << std::endl;
-
-
       Score_i_2 = X_M_i.transpose() * W_in * (Y_M_i - pi);
       Score_i = Score_i + Score_i_2;
       F_i_2 = X_M_i.transpose() * (W_in) * (D.transpose() * X_M_i);
       F_i = F_i + F_i_2;
       LogLik = LogLik + (Y_M_i.transpose().eval()*VectorXd(pi.array().log())) + ( (1 - Y_M_i.sum()) * std::log(1 - pi.sum()) );
 
+      // MatrixXd pi_mat = pi;
+      // MatrixXd pi_mat1 = pi_mat.transpose();
+      // // VectorXd pi_vec1 = pi_mat1;
+      //
+      // VectorXd pi_vec1(Map<VectorXd>(pi_mat1.data(), pi_mat1.cols()*pi_mat1.rows()));
+
       pi_ma.row(i) = pi.transpose();
+      // pi_ma.row(i) = pi_vec1;
+      // pi_ma.row(i) = pi;
+
+      // Rcout << "W_in" << std::endl;
+      // Rcout << pi.rows() << std::endl;
+      // Rcout << pi_ma.cols() << std::endl;
+      // Rcout << pi << std::endl;
+      // Rcout << pi_ma << std::endl;
 
     }
 
-    VectorXd Ones1 = VectorXd::Ones(pi_ma.rows());
-    pi_ma.col(Q) = Ones1 - pi_ma.rowwise().sum() ;
+    // VectorXd pima3 = pi_ma.rowwise().sum();
+    // pi_ma.resize(pi_ma.rows(), K);
+    // VectorXd Ones1 = VectorXd::Ones(pi_ma.rows());
+    // pi_ma.col(Q) = Ones1 - pima3 ;
 
     // To stop when LogLik is smaller than the previous
     if(iteration>1){
@@ -246,8 +258,6 @@ List GLMcat(Formula formula,
         break;
       // iteration = 25;
     }
-
-
     LogLikIter.conservativeResize(iteration+2, 1);
     LogLikIter(iteration+1) = LogLik;
     Stop_criteria = (abs(LogLikIter(iteration+1) - LogLikIter(iteration))) / (epsilon + (abs(LogLikIter(iteration+1)))) ;
@@ -264,9 +274,6 @@ List GLMcat(Formula formula,
     BETA = BETA + (F_i.inverse() * Score_i);
     // check_tutz = ((BETA - beta_old).norm())/(beta_old.norm()+check_tutz);
     iteration = iteration + 1;
-
-
-
     // if (iteration == 30) {
     //   cout << "Max iter" << endl;
     //   Rcpp::stop("Max iter");
@@ -278,12 +285,17 @@ List GLMcat(Formula formula,
     // Rcout << "LogLik" << std::endl;
     // Rcout << LogLik << std::endl;
   }
+  // Rcout << "pi_ma" << std::endl;
+  // Rcout << pi_ma << std::endl;
+  VectorXd pima3 = pi_ma.rowwise().sum();
+  MatrixXd pimadef = pi_ma;
+  pimadef.conservativeResize(pi_ma.rows(), K);
+  VectorXd Ones1 = VectorXd::Ones(pi_ma.rows());
+  pimadef.col(Q) = Ones1 - pima3 ;
+  // pimadef.block(0,Q,pi_ma.rows(),1) = Ones1 - pima3 ;
 
-  // cov_beta = (((X_EXT.transpose() * F_i_final) * X_EXT).inverse());
-
-
-  // CompleteOrthogonalDecomposition<MatrixXd> cqr(F_i_final);
-  // MatrixXd pinv = cqr.pseudoInverse();
+  // Rcout << "pimadef" << std::endl;
+  // Rcout << pimadef << std::endl;
 
   cov_beta = F_i_final.inverse();
   Std_Error = cov_beta.diagonal();
@@ -291,19 +303,6 @@ List GLMcat(Formula formula,
 
   std::vector<std::string> text=as<std::vector<std::string> >(explanatory_complete);
   std::vector<std::string> level_text=as<std::vector<std::string> >(levs1);
-  // StringVector names(Q*P_c + P_p);
-  // if(P_c > 0){
-  //   for(int var = 0 ; var < explanatory_complete.size() ; var++){
-  //     for(int cat = 0 ; cat < Q ; cat++){
-  //       names[(Q*var) + cat] = dist1.concatenate(text[var], level_text[cat]);
-  //     }
-  //   }
-  // }
-  // if(P_p > 0){
-  //   for(int var_p = 0 ; var_p < proportional.size() ; var_p++){
-  //     names[(Q*P_c) + var_p] = proportional[var_p];
-  //   }
-  // }
 
   StringVector names;
   if(threshold == "equidistant"){
@@ -342,19 +341,14 @@ List GLMcat(Formula formula,
   NumericMatrix coef = wrap(BETA);
   rownames(coef) = names;
   int df = coef.length();
-
-  // IC
-  // double AIC = (-2*LogLik) + (2 *coef.length());
-  // double BIC = (-2*LogLik) + (coef.length() * log(N) );
-
   MatrixXd predict_glmcated = X_EXT * BETA;
 
   VectorXd Ones2 = VectorXd::Ones(Y_init.rows());
   VectorXd vex1 = (Y_init.rowwise().sum()) ;
   Y_init.conservativeResize( Y_init.rows(), K);
   Y_init.col(Q) = (vex1 - Ones2).array().abs() ;
-  MatrixXd residuals = Y_init - pi_ma;
-  VectorXd pi_ma_vec(Map<VectorXd>(pi_ma.data(), pi_ma.cols()*pi_ma.rows()));
+  MatrixXd residuals = Y_init - pimadef;
+  VectorXd pi_ma_vec(Map<VectorXd>(pimadef.data(), pimadef.cols()*pimadef.rows()));
   VectorXd Y_init_vec(Map<VectorXd>(Y_init.data(), Y_init.cols()*Y_init.rows()));
   VectorXd div_arr = Y_init_vec.array() / pi_ma_vec.array();
   VectorXd dev_r(Y_init.rows());
@@ -571,7 +565,8 @@ RCPP_MODULE(GLMcatmodule){
                                _["ref_category"] = CharacterVector::create(NA_STRING),
                                _["freedom_degrees"] = R_NaN,
                                _["threshold"] = "standard",
-                               _["beta_init"] = R_NaN
+                               _["beta_init"] = NumericVector::create(NA_REAL)
+                                // _["beta_init"] = R_NaN
                  ),
                  "GLMcat models");
   Rcpp::function("predict_glmcat", &predict_glmcat,
